@@ -18,26 +18,24 @@
 #include "about.h"
 
 #include "mapview/mapview.h"
+#include "mapview/maplayer.h"
+#include "mapview/maproute.h"
 #include "mapview/mapnode.h"
 #include "completer.h"
 #include "mobileui.h"
 #include "flickcharm.h"
 
-//#if defined(Q_WS_MAEMO_5)
-//#include "fullscreenexitbutton.h"
-//#endif
-
-QString VERSION=" 0.5.1";
+QString VERSION=" 0.5.2";
 QString URL_MAPS="http://pmetro.su/Maps.html";
 QString URL_TRANSLATE="http://translate.google.com/translate?sl=ru&tl=%1&u=%2";
 
 char ICONSIZE=24; // begin size 100% scale
 QString TOOLBAR_STYLE=
-  "QToolButton,QToolButton:pressed,QLabel,QLineEdit{border:2px solid grey;border-radius:5px;background-color:qlineargradient(x1:0,y1:0,x2:0,y2:1,stop:0 #dadbde,stop:1 #f6f7fa);} "
+  "QToolButton,QToolButton:pressed,QLabel,QLineEdit{border:2px solid grey;border-radius:5px;background-color:qlineargradient(x1:0,y1:0,x2:0,y2:1,stop:0 #dadbde,stop:1 rgba(255,255,255,60%));} "
   "QToolButton{height:%1px;width:%1px;background-color:transparent;margin-top:4px;margin-left:4px;} "
   "QToolButton:pressed{margin-top:5px;margin-left:5px;} "
   "QLabel{color:black;font-weight:bold;padding:2px;} "
-  "QLineEdit{height:%1px;font-weight:bold;margin-top:2px;margin-left:2px;} ";
+  "QLineEdit{height:%1px;font-weight:bold;margin-top:2px;margin-left:2px;}";
 
 MainWindow *mainWnd;
 
@@ -55,7 +53,7 @@ QVector <QString> IniGroupStr;
 QVector <QString> IniKeyStr;
 
 MainWindow::MainWindow(QWidget *parent) :
-    QMainWindow(parent), ui(new Ui::MainWindow),Completer(NULL),SettingWindow(NULL)
+    QMainWindow(parent),map(NULL),SettingWindow(NULL),ui(new Ui::MainWindow),Completer(NULL)
 {
 #if defined(Q_OS_WINCE) || defined(Q_OS_SYMBIAN) || defined(Q_WS_MAEMO_5) || defined(Q_OS_ANDROID)
  MobileVersion=1;
@@ -64,6 +62,7 @@ MainWindow::MainWindow(QWidget *parent) :
 #endif
 
  //MobileVersion=1;
+ codePage = QTextCodec::codecForName("windows-1251");
  mainWnd=this;
  IniGroupStr << "MapView"<<"Main"<<"Other";
  IniKeyStr << "StationCustomFont"<<"StationFont"<<"StationBack"<<"StationShadow"<<"StationBackColor"<<"StationShadowColor"<<
@@ -75,12 +74,22 @@ MainWindow::MainWindow(QWidget *parent) :
  Config.appFont=qApp->font();
  appPath=qApp->applicationDirPath()+"/";
  appConfigFile=cachePath="";
+#ifdef Q_OS_LINUX
+ QString home=qgetenv("HOME");
+ appPath="/usr/share/qmetro/";
+ if(!home.isEmpty())
+ {
+  cachePath=home+"/.cache/qMetro/";
+  appConfigFile=home+"/.config/qMetro/config.conf";
+ }
+#endif
 #ifdef Q_WS_MAEMO_5
  setAttribute(Qt::WA_Maemo5AutoOrientation);
  setAttribute(Qt::WA_Maemo5StackedWindow);
+ setAttribute(Qt::WA_Maemo5NonComposited);
  appPath="/home/user/MyDocs/Data/qMetro/";
  cachePath=appPath+".cache/";
- appConfigFile="/home/user/qMetro.ini";
+ appConfigFile="/home/user/.config/qMetro/config.conf";
  ICONSIZE=64;
 #endif
 #ifdef Q_OS_ANDROID
@@ -98,10 +107,13 @@ MainWindow::MainWindow(QWidget *parent) :
  Config.appIconSize=ICONSIZE;
 
  QSettings *ini=new QSettings(appConfigFile,QSettings::IniFormat);
+ ini->setIniCodec(codePage);
  QString key;
  ini->beginGroup("Main");
  Config.locale=ini->value("Locale","").toString();
  Config.dockWidth=ini->value("DockWidth",0).toInt();
+ key="WindowFullScreen";
+ Config.appFullscreen=ini->value(key,0).toBool();
  key="MapZoomFactor";
  Config.mapZoomFactor=ini->value(key).toFloat();
  Config.mapZoomFactorCheck=ini->contains(key);
@@ -117,26 +129,37 @@ MainWindow::MainWindow(QWidget *parent) :
   qApp->setFont(Config.appFont);
  }
  TOOLBAR_STYLE=ini->value("Styles",TOOLBAR_STYLE).toString();
-
  if(Config.locale.isEmpty()) Config.locale=QLocale::system().name().section("_",0,0);
-
-
- ui->setupUi(this);
-
+ toggleLang(Config.locale);
+ ui->setupUi(this); // UI #############################
+ restoreState(QByteArray::fromHex(ini->value("WindowState").toByteArray()));
+ if(!MobileVersion)
+ {
+  key="WindowGeometry";
+  if(ini->contains(key))
+  {
+   restoreGeometry(QByteArray::fromHex(ini->value(key).toByteArray()));
+   if(windowState()==Qt::WindowFullScreen) setWindowState(Qt::WindowMaximized);
+  }
+  else
+  {
+   resize(640,480);
+   setWindowState(Qt::WindowMaximized);
+  }
+ }
  ui->mSaveScale->setChecked(Config.mapZoomFactorCheck);
  ui->mSavePosition->setChecked(Config.mapPositionCheck);
  if(!Config.dockWidth) Config.dockWidth=ui->dock->width();
  ui->dock->setVisible(0);
  //ui->eSearch->setText(tr("Find"));
  //ui->eSearch->installEventFilter(this);
+ ini->endGroup();
 
  setWindowTitle(windowTitle()+VERSION);
 
- toggleLang(Config.locale);
  map = new TMapView(appPath,cachePath);
  map->config.MobileVersion=MobileVersion;
  map->config.locale=Config.locale;
- //if(MobileVersion) map->setStyleSheet(TOOLBAR_STYLE.arg(Config.appIconSize));
 
  connect(map,SIGNAL(trackText(QString)),ui->textDescription,SLOT(setText(QString)));
  if(!MobileVersion)
@@ -152,20 +175,52 @@ MainWindow::MainWindow(QWidget *parent) :
   connect(&charm,SIGNAL(finished()),map,SLOT(mapStopDragging()));
  }
 
+
  ui->toolBar->setIconSize(QSize(Config.appIconSize,Config.appIconSize));
- QSplitter *splitter = new QSplitter;
- splitter->addWidget(ui->dock);
- splitter->addWidget(map);
- splitter->setStretchFactor(0, 0);
- splitter->setStretchFactor(1, 1);
- splitter->setChildrenCollapsible(0);
- splitter->setOpaqueResize(0);
- setCentralWidget(splitter);
 
+ if(!MobileVersion)
+ {
+  QHBoxLayout *EditLayout = new QHBoxLayout;
+  ui->bClear->setCursor(Qt::ArrowCursor);
+  ui->bClear->setStyleSheet("QToolButton{border:0px;background-color:transparent;}");
+  EditLayout->setAlignment(Qt::AlignRight);
+  EditLayout->setMargin(3);
+  EditLayout->addWidget(ui->bClear);
+  ui->eSearch->setLayout(EditLayout);
 
- loadSettings();
+  QSplitter *splitter = new QSplitter;
+  splitter->addWidget(ui->dock);
+//  QLabel *LoadingLabel =new QLabel(this);
+//  QMovie *LoadingMovie =new QMovie(":/img/loading.gif");
+//  LoadingLabel->setAlignment(Qt::AlignCenter);
+//  LoadingLabel->setMovie(LoadingMovie);
+//  LoadingMovie->start();
+  splitter->addWidget(map);
+  splitter->setStretchFactor(0, 0);
+  splitter->setStretchFactor(1, 1);
+  splitter->setChildrenCollapsible(0);
+  splitter->setOpaqueResize(0);
+  setCentralWidget(splitter);
 
+  QLabel *label = new QLabel("");
+  label->setMinimumWidth(0);
+  label->setMinimumHeight(0);
+  label->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
+  ui->toolBar->addWidget(label);
+  ui->toolBar->addAction(ui->mPreference);
 
+ }else
+ {
+  setFixedWidth(qApp->desktop()->width());
+  setFixedHeight(qApp->desktop()->height());
+  centralWidget()->layout()->addWidget(map);
+  mUi = new TMobileUi(this,map);
+  //setCentralWidget(map);
+ }
+
+#ifdef Q_OS_LINUX
+ ui->menuHelp->setIcon(QIcon());
+#endif
 #ifdef Q_OS_SYMBIAN
  // Remove context menu from all widgets
  QWidgetList widgets=QApplication::allWidgets();
@@ -180,21 +235,16 @@ MainWindow::MainWindow(QWidget *parent) :
  //ui->menuHelp->addAction(ui->menuLanguage->menuAction());
 #endif
 
- QLabel *label = new QLabel("");
- label->setMinimumWidth(0);
- label->setMinimumHeight(0);
- label->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
- ui->toolBar->addWidget(label);
- ui->toolBar->addAction(ui->mPreference);
 
- if(MobileVersion)
- {
-  setFixedWidth(qApp->desktop()->width());
-  setFixedHeight(qApp->desktop()->height());
-  mUi = new TMobileUi(this,map);
- }
+ // apply config
+ map->show();
+ if(Config.appFullscreen) //AppConfig.value(igMain).value(ikWFullScreen).toInt()
+      ui->mFullScreen->setChecked(1);
+ else (MobileVersion ? showMaximized():show());
 
- ui->dock->resize(QSize(Config.dockWidth,ui->dock->height()));
+ //map->resize(qApp->desktop()->width(),qApp->desktop()->availableGeometry().height()-(height()-map->height())-20);
+ map->resize(geometry().right()-geometry().left(),geometry().bottom()-geometry().top()-(height()-map->height()));
+ loadSettings();
 
  if(Config.mapZoomFactorCheck) // first
  {
@@ -209,12 +259,6 @@ MainWindow::MainWindow(QWidget *parent) :
   map->verticalScrollBar()->setValue(Config.mapPosition.y());
  }
 
- if(AppConfig.value(igMain).value(ikWFullScreen).toInt())
-      ui->mFullScreen->setChecked(1);
- else showMaximized();
-
- map->show();
-
  connect(ui->cbDelay,SIGNAL(currentIndexChanged(int)),map,SLOT(setDelayIndex(int)));
  connect(ui->sbDelay,SIGNAL(valueChanged(int)),map,SLOT(setDelayMetro(int)));
 
@@ -222,7 +266,19 @@ MainWindow::MainWindow(QWidget *parent) :
  connect(ui->mZoomOut,SIGNAL(triggered()),map,SLOT(zoomOut()));
  connect(ui->mFit,SIGNAL(triggered()),map,SLOT(fitMap()));
 
- connect(map->nodeAInfo, SIGNAL(triggered()),this, SLOT(nodeInformation()));
+ map->nodePopup = new QMenu(this);
+ map->nodePopup->addAction(new QAction(QIcon(":/img/dialog-information.png"),tr("Information"),this));
+ map->nodePopup->addSeparator();
+ map->nodePopup->addAction(tr("Start"));
+ map->nodePopup->addAction(tr("Finish"));
+ map->nodePopup->addSeparator();
+ map->nodePopup->addAction(tr("Default start"));
+ map->nodePopup->actions().at(5)->setCheckable(1);
+ connect(map->nodePopup, SIGNAL(aboutToShow()),this, SLOT(checkNodeDefault()));
+ connect(map->nodePopup->actions().at(0), SIGNAL(triggered()),this, SLOT(nodeInformation()));
+ connect(map->nodePopup->actions().at(2),SIGNAL(triggered()),map,SLOT(setNodeBegin()));
+ connect(map->nodePopup->actions().at(3),SIGNAL(triggered()),map,SLOT(setNodeEnd()));
+ connect(map->nodePopup->actions().at(5),SIGNAL(triggered(bool)),this,SLOT(setNodeDefault(bool)));
 
  if(map->nodeByID.count()) Completer= new TCompleter(map,ui->eSearch);
 
@@ -250,6 +306,7 @@ MainWindow::MainWindow(QWidget *parent) :
   connect(langAction, SIGNAL(triggered()),this, SLOT(toggleLang()));
  }
  if(MobileVersion) toggleLang(Config.locale);
+ loadNodeDefault();
 }
 
 MainWindow::~MainWindow()
@@ -257,6 +314,49 @@ MainWindow::~MainWindow()
  delete ui;
 }
 
+void MainWindow::setNodeDefault(bool checked,TMapNode *Node)
+{
+ if(!Node) Node=map->nodeClick;
+ if(!Node) return;
+ //qDebug() << checked << map->params.mapName << Node->Route->DisplayName << Node->DisplayName;
+ QSettings ini(appConfigFile,QSettings::IniFormat);
+ ini.setIniCodec(codePage);
+ QString key;
+ ini.beginGroup("DefaultStation");
+ if(checked)
+ {
+  Config.mapDefaultStationID=Node->ID;
+  ini.setValue(map->params.mapName,Node->Route->DisplayName+","+Node->DisplayName);
+  map->setNodeBegin(Node);
+ }
+ else
+  ini.remove(map->params.mapName);
+ ini.endGroup();
+}
+void MainWindow::loadNodeDefault()
+{
+ Config.mapDefaultStationID=-1;
+ QSettings ini(appConfigFile,QSettings::IniFormat);
+ ini.setIniCodec(codePage);
+ QStringList DefaultStation=ini.value("DefaultStation/"+map->params.mapName).toString().split(",");
+ //qDebug() << DefaultStation;
+ if(DefaultStation.count()==2)
+ {
+  TMapNode *Node=NULL;
+  TMapRoute *Route=map->layer.first()->findRoute(DefaultStation.first());
+  if(Route) Node=Route->findNode(DefaultStation.last());
+  if(Node)
+  {
+   Config.mapDefaultStationID=Node->ID;
+   map->setNodeBegin(Node);
+  }
+ }
+}
+void MainWindow::checkNodeDefault()
+{
+ map->nodePopup->actions().at(5)->setChecked((map->nodeClick->ID==Config.mapDefaultStationID));
+}
+
 //bool MainWindow::eventFilter(QObject *obj, QEvent *ev)
 //{
 // if(obj==ui->eSearch && ui->eSearch->isVisible() && ev->type()==QEvent::FocusIn)
@@ -286,6 +386,7 @@ bool MainWindow::eventFilter(QObject *obj, QEvent *e)
  case QEvent::Resize:
   {
    if(map->selectedNode.count()) map->selectedNodesToFront(1);
+   map->mapStartDragging();
    //map->fitMap();
    return 1;
   }
@@ -299,29 +400,23 @@ bool MainWindow::eventFilter(QObject *obj, QEvent *e)
    {
     //if(windowState()==Qt::WindowFullScreen)
     ui->mFullScreen->setChecked(0); //showMaximized();
+    map->clearTrack();
    }
+   if(key==Qt::Key_F1) on_mAbout_triggered();
    if(key==Qt::Key_F11)
    {
     ui->mFullScreen->setChecked(windowState()!=Qt::WindowFullScreen);
    }
-
+   QString char_key=event->text().simplified();
+   if(!ui->mFind->isChecked() && !char_key.isEmpty())//((key>=Qt::Key_A && key<=Qt::Key_Z)||(key>=Qt::Key_0 && key<=Qt::Key_9)) )
+   {
+    ui->mFind->setChecked(1);
+    ui->eSearch->setText(char_key);
+    ui->eSearch->setFocus();
+   }
    return 1;
   }
   break;
-  //    case QEvent::FocusIn:
-  //        {
-  //         if(obj==ui->eSearch)
-  //         {
-  //          ((QLineEdit *)obj)->selectAll();
-  //          return 0;
-  //         }
-  //         else
-  //         if(e->type()==QEvent::MouseButtonPress || e->type()==QEvent::MouseButtonRelease)  return 1;
-
-
-
-  //        }
-  //        break;
  default: ;
  }
 
@@ -353,7 +448,7 @@ void MainWindow::on_mOpen_triggered()
 #ifdef Q_OS_WINCE
  options=QFileDialog::DontUseNativeDialog;
 #endif
- QString MapPath=QString("%1Map/").arg(appPath);
+ QString MapPath=QString("%1map/").arg(appPath);
 
 // ZIPed maps will be unpacked
  QDir Dir(MapPath);
@@ -381,7 +476,7 @@ void MainWindow::on_mOpen_triggered()
  if(Completer) delete Completer;
  map->loadMap(fileName);
  map->fitMap();
- //map->viewport()->update();
+ //map->viewport()->repaint();
  ui->eSearch->setText("");
  Completer= new TCompleter(map,ui->eSearch);
 //qDebug() << "Load time: " << testTimer.elapsed();
@@ -408,11 +503,17 @@ void MainWindow::on_eSearch_returnPressed()
 
 void MainWindow::on_mFind_toggled(bool checked)
 {
- if(!MobileVersion)
+ if(!MobileVersion) // PC
  {
-  if(!checked) Config.dockWidth=ui->dock->width();
+  map->mapStartDragging();
+  if(checked)
+   ui->dock->resize(QSize(Config.dockWidth,ui->dock->height()));
+  else
+   Config.dockWidth=ui->dock->width();
   ui->dock->setVisible(checked);
- }else
+  if(checked) ui->dock->repaint();
+ }
+ else
  {
   //map->setUpdatesEnabled(0);
   ui->eSearch->setVisible(checked);
@@ -442,9 +543,8 @@ void MainWindow::setDelayNames(QStringList list)
 void MainWindow::on_mAbout_triggered()
 {
  About *a=new About(this);
- if(MobileVersion)
-      a->showMaximized();
- else a->exec();
+ //if(MobileVersion) a->showMaximized(); else
+ a->exec();
 }
 
 void MainWindow::on_mPreference_triggered()
@@ -481,6 +581,7 @@ void MainWindow::toggleLang(QString locale)
 
  qApp->installTranslator(&appTranslator);
  Config.locale=locale;
+ if(map) map->config.locale=locale;
 }
 
 void infoAddTab(QWidget *widget, const QString &Caption,const QString &Text)
@@ -532,7 +633,6 @@ void MainWindow::nodeInformation()
 
  TMapView *MapScheme;
  QVBoxLayout *layout;
- QToolBar *Bar;
  QTabWidget *Tabs=DetailsWindow->ui->tabs;
  QString AllText="";
  foreach(QString key,map->nodeClick->Info.keys())
@@ -567,9 +667,11 @@ void MainWindow::nodeInformation()
 
     map->extractFiles(map->params.mapFile,FileName,map->params.mapPath);
     MapScheme->loadMapImageToPaintCmd(FileName,MapScheme->paintBkg);
+    MapScheme->horizontalScrollBar()->setValue(MapScheme->horizontalScrollBar()->maximum()/2);
+    MapScheme->verticalScrollBar()->setValue(MapScheme->verticalScrollBar()->maximum()/2);
     MapScheme->state.mapLoaded=1;
     Tabs->addTab(MapScheme,key);
-    TMobileUi *toolbars = new TMobileUi(this,MapScheme,1);
+    new TMobileUi(this,MapScheme,1);
    }
   }else
    AllText+=QString("<h3>%1</h3><br>%2<br>").arg(key).arg(Info.Value);
@@ -603,24 +705,30 @@ void MainWindow::closeEvent(QCloseEvent * e)
 {
  if(!isVisible()) { e->accept(); return;} // FIX: double exec dont know why
  map->state.mapLoaded=0;
- setVisible(0);
- QSettings cfg(appConfigFile,QSettings::IniFormat);
- QString key;
- cfg.beginGroup("Main");
- cfg.setValue("MapLastFile",map->params.mapFile);
- cfg.setValue("Locale",Config.locale);
  if(ui->dock->isVisible()) Config.dockWidth=ui->dock->width();
- cfg.setValue("DockWidth",Config.dockWidth);
+ setVisible(0);
+ QSettings ini(appConfigFile,QSettings::IniFormat);
+ ini.setIniCodec(codePage);
+ QString key;
+ ini.beginGroup("Main");
+ ini.setValue("WindowState", saveState().toHex());
+ if(!MobileVersion) ini.setValue("WindowGeometry", saveGeometry().toHex());
+ ini.setValue("MapLastFile",map->params.mapFile);
+ key="Locale";
+ if(ini.contains(key))
+  ini.setValue(key,Config.locale);
+ else
+  if(Config.locale!=QLocale::system().name().section("_",0,0)) ini.setValue(key,Config.locale);
+ ini.setValue("DockWidth",Config.dockWidth);
  key="MapZoomFactor";
  if(ui->mSaveScale->isChecked())
-      cfg.setValue(key,map->params.mapZoomFactor);
- else cfg.remove(key);
+  ini.setValue(key,QString::number(map->params.mapZoomFactor));
+ else ini.remove(key);
  key="MapPosition";
  if(ui->mSavePosition->isChecked())
-      cfg.setValue(key,QPoint(map->horizontalScrollBar()->value(),map->verticalScrollBar()->value()));
- else cfg.remove(key);
- if(MobileVersion) cfg.setValue("Styles",TOOLBAR_STYLE);
- cfg.endGroup();
+      ini.setValue(key,QPoint(map->horizontalScrollBar()->value(),map->verticalScrollBar()->value()));
+ else ini.remove(key);
+ if(MobileVersion) ini.setValue("Styles",TOOLBAR_STYLE);
  if(AppConfig.value(igOther).value(ikMCleanCache).toInt()) removeDir(cachePath);
  else
  if(map->config.enableCaching && !QFile::exists(map->params.mapPath+".cache"))
@@ -681,19 +789,19 @@ void applySettings(const IniSettingMap &settings)
   }
  }
 
+ map->applyConfig(); // + load last map
+
+ // change params
  if(map->config.enableCustomFont)
       map->params.nodeFont=nodeFont;
  else map->params.nodeFont=map->params.nodeFontDefault;
 
- map->applyConfig(); // + load last map
-
- //map->show();
 }
 
 void saveSettings(const IniSettingMap &settings)
 {
  QSettings cfg(appConfigFile,QSettings::IniFormat);
-
+ cfg.setIniCodec(mainWnd->codePage);
  if(AppConfig.count())
  {
   QMapIterator<IniGroup,IniKeyMap> i(AppConfig);
@@ -739,6 +847,7 @@ void loadSettings()
  map->defaultConfig();
 
  QSettings cfg(appConfigFile,QSettings::IniFormat);
+ cfg.setIniCodec(mainWnd->codePage);
 
  foreach(QString group, cfg.childGroups())
  {
@@ -803,7 +912,7 @@ qint64 sizeOfDir(const QString &DirName)
 
 void MainWindow::on_mAddMap_triggered()
 {
- QString MapPath=appPath+"Map/";
+ QString MapPath=appPath+"map/";
  if(Config.locale!="ru")
       QDesktopServices::openUrl(QUrl(URL_TRANSLATE.arg(Config.locale).arg(URL_MAPS)));
  else QDesktopServices::openUrl(QUrl(URL_MAPS));
