@@ -38,6 +38,7 @@ public:
 
     //void showEvent(QShowEvent * event);
 
+    QTextCodec *codePage;
     TMapView *map;
     Settings *SettingWindow;
 
@@ -50,10 +51,12 @@ public:
      bool mapZoomFactorCheck;
      bool mapPositionCheck;
      bool mapKineticScrollCheck;
+     int mapDefaultStationID;
 
      QFont appFont;
      qreal appUiScale;
      int appIconSize;
+     bool appFullscreen;
 
      QSize mainWindow;
      QSize settingWindow;
@@ -77,6 +80,9 @@ public slots:
     void setScaleForUi(qreal UiScale);
     void setScaleForUi(int UiScale) { setScaleForUi((qreal)(UiScale/100.0)); }
     void toggleLang(QString locale="");
+    void setNodeDefault(bool checked,TMapNode *Node=NULL);
+    void loadNodeDefault();
+    void checkNodeDefault();
 
 private slots:
 
