@@ -41,17 +41,16 @@ void TMapNode::init()
 {
  int sWidth=Map->params.nodeWidth;
  QRectF r;
- bool leftAlign=0;
  QPixmap pix(1,1);
  QPainter p(&pix); // fake
  p.setFont(Map->params.nodeFont);
 
-
  DisplayName=(Alias.isEmpty() ? Name:Alias);
  if(Map->config.enableTranslit) Map->convertToTranslit(DisplayName);
  if(!Map->config.MobileVersion) setToolTip(DisplayName);
 
- NRect=QRect(Rect.topLeft()-pos().toPoint()-QPoint(1,0),QSize(Rect.width(),Rect.height()));
+ if(Rect!=QRect()) NRect=QRect(Rect.topLeft()-pos().toPoint()-QPoint(1,0),QSize(Rect.width(),Rect.height()));
+ else NRect=Rect;
  EllipseRect=QRect(QPoint(-sWidth,-sWidth),QPoint(sWidth,sWidth));
  Align=Map->params.nodeFontStyle;
  Vertical=0;
@@ -85,6 +84,33 @@ void TMapNode::init()
   else Align+=Qt::AlignRight;
   p.drawText(NRect,Align,DisplayName,&NRect);
  }
+
+////////////////////////////////////////////
+// TextLayout with setCacheEnabled(1) more perfomance but trouble...
+//
+// TextLayout = new QTextLayout(DisplayName,Map->params.nodeFont);
+// QTextOption option;
+// option.setAlignment(Qt::Alignment(Align));
+// option.setUseDesignMetrics(1);
+// //option.setWrapMode(QTextOption::WordWrap);
+// TextLayout->setTextOption(option);
+// //TextLayout->setFont();
+//
+// qreal vertPos=0;
+// QTextLine line;
+// TextLayout->beginLayout();
+// line=TextLayout->createLine();
+// while (line.isValid())
+// {
+//  line.setLineWidth(NRect.width());
+//  line.setPosition(QPointF(0, vertPos));
+//  vertPos+=line.height()-2;
+//  line=TextLayout->createLine();
+// }
+// TextLayout->endLayout();
+// TextLayout->setCacheEnabled(1);
+////////////////////////////////////////////
+
  r=shape().boundingRect();
  prepareGeometryChange();
  NodeWidth=r.width();
@@ -124,7 +150,7 @@ QPainterPath TMapNode::shape() const
 }
 
 void TMapNode::paint(QPainter *p, const QStyleOptionGraphicsItem *option, QWidget *)
-{
+{Q_UNUSED(option);
  if(!Map->state.mapLoaded) return;
 
  if(Map->config.enableCaching && Map->state.mapCached)
@@ -144,7 +170,7 @@ void TMapNode::paint(QPainter *p, const QStyleOptionGraphicsItem *option, QWidge
  if(TypeNode==ntConstruct)
  {
   p->setPen(QPen(Route->color,2));
-  p->setBrush(Qt::white);
+  p->setBrush(Map->params.mapColorBack);
  }else
  if(Map->nodeBeginID==ID)
  {
@@ -177,6 +203,8 @@ void TMapNode::paint(QPainter *p, const QStyleOptionGraphicsItem *option, QWidge
  {
   QRectF TextRect;
   p->setFont(Map->params.nodeFont);
+//  if(TextLayout->font()!=Map->params.nodeFont)
+//   TextLayout->setFont(Map->params.nodeFont);
   p->setBrush(Route->labelsBColor);
 
   if(Vertical)
@@ -194,15 +222,19 @@ void TMapNode::paint(QPainter *p, const QStyleOptionGraphicsItem *option, QWidge
    }
   }
   else // backcolor
-  if(Route->labelsBColor.alpha()) p->setBackgroundMode(Qt::OpaqueMode);
+  if(Route->labelsBColor.alpha())
+  {
+   p->setBackgroundMode(Qt::OpaqueMode);
+   p->setBackground(Map->params.mapColorBack);
+  }
   else // shadow
   {
    p->setBackgroundMode(Qt::TransparentMode);
-   p->setPen(Qt::white);
-   p->drawText(TextRect.adjusted(qreal(-0.2),qreal(-0.2),qreal(-0.2),qreal(-0.2)),Align,DisplayName);
+   p->setPen(Map->params.mapColorBack);
+   p->drawText(TextRect.adjusted(qreal(-0.6),qreal(-0.6),qreal(-0.6),qreal(-0.6)),Align,DisplayName);
 
    p->setPen(Route->color.dark());
-   p->drawText(TextRect.adjusted(qreal(0.3),qreal(0.3),qreal(0.3),qreal(0.3)),Align,DisplayName);
+   p->drawText(TextRect.adjusted(qreal(0.6),qreal(0.6),qreal(0.6),qreal(0.6)),Align,DisplayName);
   }
 
   if(Map->config.enableCustomFont && Map->config.enableStationShadowColor)
@@ -214,9 +246,12 @@ void TMapNode::paint(QPainter *p, const QStyleOptionGraphicsItem *option, QWidge
   }
 
   p->setPen(Route->color);
+
   if(Vertical)
        p->drawText(NRect90,Align,DisplayName,&NRect90);
   else p->drawText(NRect,Align,DisplayName,&NRect);
+  //TextLayout->draw(p,TextRect.topLeft());
+
  }
 }
 
